package com.gyee.edge.loader.golden;


import com.gyee.edge.loader.data.TsPointData;

import java.util.List;

public interface ILatestDao {

    int writeDoubleLatest(List<TsPointData> list) throws Exception;

    int writeStringLatest(List<TsPointData> list) throws Exception;

    int writeBooleanLatest(List<TsPointData> list) throws Exception;

    int writeLongLatest(List<TsPointData> list) throws Exception;

    int writeBlobLatest(List<TsPointData> list) throws Exception;

    int writeCoordinateLatest(List<TsPointData> list) throws Exception;

    int writeFloatLatest(List<TsPointData> list) throws Exception;

    boolean writeLatest(List<TsPointData> dataList) throws Exception;

}
